package net.hostsharing.hsadminng.hs.hosting.asset.factories;

import net.hostsharing.hsadminng.hs.booking.generated.api.v1.model.HsHostingAssetAutoInsertResource;
import net.hostsharing.hsadminng.hs.booking.generated.api.v1.model.HsHostingAssetSubInsertResource;
import net.hostsharing.hsadminng.hs.booking.generated.api.v1.model.HsHostingAssetTypeResource;
import net.hostsharing.hsadminng.hs.booking.item.HsBookingItemRealEntity;
import net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAsset;
import net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetRealEntity;
import net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetType;
import net.hostsharing.hsadminng.hs.office.contact.HsOfficeContactRealEntity;
import net.hostsharing.hsadminng.mapper.StandardMapper;
import net.hostsharing.hsadminng.persistence.EntityManagerWrapper;

import jakarta.validation.ValidationException;
import java.net.IDN;
import java.util.List;
import java.util.function.Function;

import static net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetType.DOMAIN_HTTP_SETUP;
import static net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetType.DOMAIN_MBOX_SETUP;
import static net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetType.DOMAIN_SMTP_SETUP;

public class DomainSetupHostingAssetFactory extends HostingAssetFactory {

    public DomainSetupHostingAssetFactory(
            final EntityManagerWrapper emw,
            final HsBookingItemRealEntity newBookingItemRealEntity,
            final HsHostingAssetAutoInsertResource asset,
            final StandardMapper standardMapper) {
        super(emw, newBookingItemRealEntity, asset, standardMapper);
    }

    @Override
    protected HsHostingAsset create() {
        final var domainSetupAsset = createDomainSetupAsset(getDomainName());

        // TODO.legacy: as long as we need to be compatible, we always do all technical domain-setups
        final var subHostingAssetResources = getSubHostingAssetResources();
        final var domainHttpSetupAsset = createDomainHttpSetupAssetEntity(
                subHostingAssetResources,
                getDomainName(),
                domainSetupAsset);
        final var assignedToUnixUserAsset = domainHttpSetupAsset.getAssignedToAsset();

        // TODO.legacy: don't create DNS setup as long as we need to remain support legacy web-ui etc.

        createDomainSubSetupAssetEntity(
                domainSetupAsset,
                DOMAIN_MBOX_SETUP,
                builder -> builder
                        .assignedToAsset(assignedToUnixUserAsset.getParentAsset())
                        .identifier(getDomainName() + "|MBOX")
                        .caption("HTTP-Setup für " + IDN.toUnicode(getDomainName())));

        createDomainSubSetupAssetEntity(
                domainSetupAsset,
                DOMAIN_SMTP_SETUP,
                builder -> builder
                        .assignedToAsset(assignedToUnixUserAsset.getParentAsset())
                        .identifier(getDomainName() + "|SMTP")
                        .caption("HTTP-Setup für " + IDN.toUnicode(getDomainName())));

        return domainSetupAsset;
    }

    private HsHostingAssetRealEntity createDomainSetupAsset(final String domainName) {
        return HsHostingAssetRealEntity.builder()
                .bookingItem(fromBookingItem)
                .type(HsHostingAssetType.DOMAIN_SETUP)
                .identifier(domainName)
                .caption(asset.getCaption() != null ? asset.getCaption() : domainName)
                .parentAsset(ref(HsHostingAssetRealEntity.class, asset.getParentAssetUuid()))
                .alarmContact(ref(HsOfficeContactRealEntity.class, asset.getAlarmContactUuid()))
                .subHostingAssets(
                        standardMapper.mapList(getSubHostingAssetResources(), HsHostingAssetRealEntity.class)
                )
                .build();
    }

    private HsHostingAssetRealEntity createDomainHttpSetupAssetEntity(
            final List<HsHostingAssetSubInsertResource> subHostingAssetResources,
            final String domainName,
            final HsHostingAssetRealEntity domainSetupAsset) {
        final var domainHttpSetupAssetResource = subHostingAssetResources.stream()
                .filter(ha -> ha.getType() == HsHostingAssetTypeResource.DOMAIN_HTTP_SETUP)
                .findFirst().orElseThrow(() -> new ValidationException(
                        domainName + ": missing target unix user (assignedToHostingAssetUuid) for DOMAIN_HTTP_SETUP "));
        final var domainHttpSetupAsset = domainSetupAsset.getSubHostingAssets()
                .stream()
                .filter(sha -> sha.getType() == DOMAIN_HTTP_SETUP)
                .findFirst()
                .orElseThrow();
        domainHttpSetupAsset.setParentAsset(domainSetupAsset);
        final var assignedToUnixUserAsset =
                emw.find(HsHostingAssetRealEntity.class, domainHttpSetupAssetResource.getAssignedToAssetUuid());
        domainHttpSetupAsset.setAssignedToAsset(assignedToUnixUserAsset);
        return domainHttpSetupAsset;
    }

    private void createDomainSubSetupAssetEntity(
            final HsHostingAssetRealEntity domainSetupAsset,
            final HsHostingAssetType subAssetType,
            final Function<HsHostingAssetRealEntity.HsHostingAssetRealEntityBuilder<?, ?>, HsHostingAssetRealEntity.HsHostingAssetRealEntityBuilder<?, ?>> builderTransformer) {
        final var resourceType = HsHostingAssetTypeResource.valueOf(subAssetType.name());
        if (getSubHostingAssetResources().stream().noneMatch(ha -> ha.getType() == resourceType)) {
            final var subAssetEntity = builderTransformer.apply(
                            HsHostingAssetRealEntity.builder()
                                    .type(HsHostingAssetType.valueOf(subAssetType.name()))
                                    .parentAsset(domainSetupAsset))
                    .build();
            domainSetupAsset.getSubHostingAssets().add(subAssetEntity);
        }
    }

    private String getDomainName() {
        return asset.getIdentifier();
    }

    private List<HsHostingAssetSubInsertResource> getSubHostingAssetResources() {
        return asset.getSubHostingAssets();
    }

    @Override
    protected void persist(final HsHostingAsset newHostingAsset) {
        super.persist(newHostingAsset);
        newHostingAsset.getSubHostingAssets().forEach(super::persist);
    }
}
